/**
 * Copyright (c) 2015 Impact Mobile Inc.
 */
package com.test;

import java.io.IOException;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.jms.JMSContext;
import javax.jms.JMSException;
import javax.jms.JMSProducer;
import javax.jms.Queue;
import javax.jms.TextMessage;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;

@Stateless
public class QueueProducer
{
    public static final String QUEUE = "hornetq-test.test-queue";
    public static final String QUEUE_JNDI = "java:jboss/queue/" + QUEUE;

    private static final ObjectMapper MAPPER = new ObjectMapper()
        .configure( DeserializationFeature.UNWRAP_ROOT_VALUE, false );

    @Resource( lookup = QUEUE_JNDI )
    private Queue queue;

    @Inject
    private JMSContext jmsContext;

    @TransactionAttribute( TransactionAttributeType.REQUIRES_NEW )
    public void sendJmsMessages( List<JmsMessage> msgs )
    {
        try
        {
            JMSProducer jmsProducer = jmsContext.createProducer();

            for ( JmsMessage msg : msgs )
            {
                if ( Boolean.getBoolean( "hornetq-clustering.group_id.enabled" ) )
                {
                    jmsProducer.setProperty( "_HQ_GROUP_ID", msg.getMessage() );
                }

                jmsProducer.send( queue, toJMSBody( msg ) );
            }
        }
        catch ( JsonProcessingException e )
        {
            throw new RuntimeException( String.format( "Unable to serialize probe to text: %s", e.getMessage() ) ); //$NON-NLS-1$
        }
    }

    public static String toJMSBody( Object obj ) throws JsonProcessingException
    {
        return MAPPER.writeValueAsString( obj );
    }
}
